var activeOverlay = 0;
var added = false;

$(document).ready(function() {

  initTooltips();
  initVideoChannel();
  initServiceId();

  $('#customScale').on('keyup change', function() {
    validateRange($(this), {
      minVal: 30,
      maxVal: 200,
      invalidMsg: 'Must be from interval 30-200%.',
      disableBtnId: 'saveOverlayBtn'
    });
  });

  $('#customDuration').on('keyup change', function() {
    validateRange($(this), {
      minVal: 0,
      maxVal: 99999,
      invalidMsg: 'Must be from interval 0-99999s.',
      disableBtnId: 'saveOverlayBtn'
    });
  });

  // Fill form
  Switch().init('customEnabled');

  $('#customCustomName')
    .val(serviceArrTmp[activeItem].customName || '')
    .keyup(onCustomNameKeyUp)
    .attachValidation({
      pattern: '^[A-Za-z0-9 -]{0,}$',
      invalidMsg: 'Only chars A-Z, a-z, 0-9, white space and dash are allowed.',
      disableBtnId: 'saveBtn'
    });
  const resolutionId = '#customResolution';
  const videoChannelId = '#customVideoChannel';
  createResolutionList(resolutionId);
  onResolutionChange(resolutionId, videoChannelId);
  const automationTypeId = '#customAutomationType';
  createTriggeringElement(automationTypeId, serviceArrTmp[activeItem].automationType, serviceArrTmp[activeItem].schedule);
  $(automationTypeId).change(handleAutomationType);
  onVideoChannelChange(resolutionId, videoChannelId, serviceArrTmp[activeItem].camera);
  createVideoChannelElement(videoChannelId, serviceArrTmp[activeItem].camera).then(function() {
    // Callbacks for changes
    $('#mainForm .form-control').change(formChangedCallback);
    $('#saveOverlayBtn').click(saveEditedOverlayCallback);
    $('#editOverlayModalDismiss').click(cancelEditedOverlayCallback);
    $('#cancelOverlayBtn').click(cancelEditedOverlayCallback);
    $('#addOverlayBtn').click(addOverlayClickedCallback);
    $('#customImageManagerBtn').click(showImageManagerCallback);
    $('#customImgPath').change(customImagePathChanged);

    formChangedCallback();

    let currentCameraResolutionObject = cameraResolutions[parseInt($('#customResolution').val())];
    createCameraPreview($('#customVideoChannel'), currentCameraResolutionObject, serviceArrTmp[activeItem].camera);
  });

  showOverlayList();
  createScheduleBtn();
});

function handleAutomationType(event) {
  const automation_type_value = $(event.target).val();
  $('.automationTypeId').removeClass('d-block').addClass('d-none');
  const automation_type_id = automation_type_value !== 'manual' ? 'time' : 'manual';
  $('.automationTypeId[data-type-id="' + automation_type_id + '"]').removeClass('d-none').addClass('d-block');

  handleAutomationTypeLabel(automation_type_value);
}

function formChangedCallback() {
  serviceArrTmp[activeItem].customName = $('#customCustomName').val();

  if (serviceArrTmp[activeItem].name == "custom") {
    serviceArrTmp[activeItem].name = "images";
  }
  let currentCameraResolutionObject = cameraResolutions[parseInt($('#customResolution').val())];
  serviceArrTmp[activeItem].width = currentCameraResolutionObject.width;
  serviceArrTmp[activeItem].height = currentCameraResolutionObject.height;
  serviceArrTmp[activeItem].automationType = $('#customAutomationType').val();

  serviceArrTmp[activeItem].camera = saveVideoChannelValue(serviceArrTmp[activeItem].camera, $('#customVideoChannel'));

  showSaveWarning();
  createServiceList(serviceArrTmp);
}

/** Create input option HTML
 * @param {number} id
 * @param {boolean} is_active
 * @returns {string}
 */
function get_overlay_radio_input_HTML(id, is_active) {
  const checked = is_active ? 'checked' : '';
  const label_text = is_active ? 'active' : 'choose';
  const class_active = is_active ? 'overlay_active' : '';
  const radio_input = '<input type="radio" id="overlayEnabled' + id + '" data-id="' + id + '" class="radio_overlay_active form-check-input" ' + checked + ' />';
  const radio_input_label = '<label class="radio_overlay_active_label form-check-label ' + class_active + '" for="overlayEnabled' + id + '">' + label_text + '</label>';
  return '<div class="form-group custom-control form-check mb-0">' + radio_input + radio_input_label + '</div>';
}

function RadioInput(elem) {

  const disable = () => handleDisabled(true);
  const enable = () => handleDisabled(false);
  /**
   * @param {boolean} disable
   */
  const handleDisabled = (disable) => {
    elem.disabled = disable;
  }

  const check = () => handleCheck(true);
  const uncheck = () => handleCheck(false);
  /**
   * @param {boolean} check
   */
  const handleCheck = (check) => {
    elem.checked = check;
    setLabel(check);
    handleSaveOverlay(elem, check);
  }

  /**
   * @param {boolean} check
   */
  const setLabel = (check) => {
    const label = RadioLabel($(elem).next('label'));
    label.setText(check ? 'active' : 'choose');
    label.setActive(check);
  }

  return {
    disable,
    enable,
    check,
    uncheck,
  }
}

/**
 * @param elem
 * @param {boolean} is_active
 */
function handleSaveOverlay(elem, is_active) {
  const elem_id = elem.getAttribute('data-id');
  const tmp_overlay_list = serviceArrTmp[activeItem].overlayList[elem_id];
  const overlay_list = serviceArr[activeItem].overlayList[elem_id];
  tmp_overlay_list.active = is_active;
  const exists_in_overlay_list = tmp_overlay_list === overlay_list;
  if (!exists_in_overlay_list) {
    serviceArr[activeItem].overlayList.push(tmp_overlay_list);
  }
  overlay_list.active = is_active;
}

function showOverlayList() {
  let overlayListHtml = '';
  if (serviceArrTmp[activeItem].overlayList != undefined && serviceArrTmp[activeItem].overlayList.length > 0) {
    overlayListHtml += '<tbody>';
    let overlayList = serviceArrTmp[activeItem].overlayList;
    for (let i = 0; i < overlayList.length; i++) {
      overlayListHtml += '<tr>';
      const image_preview = '<img id="imageManagerPreview" src="' + get_image_src(overlayList[i].imgPath) + '" style="max-width: 60px;" />';
      overlayListHtml += '<td style="width: 1%;">' + image_preview + '</td>';

      overlayListHtml += '<td style="max-width: 150px; overflow: hidden; text-overflow: ellipsis; white-space: nowrap;">';
      overlayListHtml += '<span style="font-style: italic;">' + overlayList[i].imgName + '</span>';
      overlayListHtml += '</td>';

      const is_automatic_automation = serviceArrTmp[activeItem].automationType !== 'manual';
      const manual_class = !is_automatic_automation ? 'd-block' : 'd-none';
      const time_class = is_automatic_automation ? 'd-block' : 'd-none';
      const input_radio_html = '<div class="automationTypeId ' + manual_class + '" data-type-id="manual">' + get_overlay_radio_input_HTML(i, overlayList[i].active) + '</div>';
      const duration_text = overlayList[i].duration ? overlayList[i].duration + 's' : '0s';
      const duration_html = '<div class="automationTypeId hidden-xs ' + time_class + '" data-type-id="time" style="text-align: center;">' + duration_text + '</div>';
      overlayListHtml += '<td>' + input_radio_html + duration_html + '</td>';

      const coord_text = coordSystemToNiceName(overlayList[i].coordSystem) + '<br />';
      const pos_text = 'x: ' + overlayList[i].pos_x + ', y: ' + overlayList[i].pos_y;
      const scale_text = !isNaN(overlayList[i].scale) ? ', <br>scale: ' + convertFloatToPercentage(overlayList[i].scale) + '%' : '';
      overlayListHtml += '<td class="hidden-xs" style="text-align: center;">' + coord_text + pos_text + scale_text + '</td>';

      overlayListHtml += '<td style="text-align: center;">';
      overlayListHtml += '<a href="#" id="moveOverlay' + i + '" class="overlayMoveUp icon formItem" style="margin-left: 0;"><span class="fas fa-arrow-circle-up" data-toggle="tooltip" title="Move Up"></span></a>';
      overlayListHtml += '<a href="#" id="moveOverlay' + i + '" class="overlayMoveDown icon formItem"><span class="fas fa-arrow-circle-down" data-toggle="tooltip" title="Move Down"></span></a>';
      overlayListHtml += '<a href="#" id="editOverlay' + i + '" class="overlayEdit formItem icon"><span class="fas fa-edit" data-toggle="tooltip" title="Edit"></span></a>';
      overlayListHtml += '<a href="#" id="removeOverlay' + i + '" class="overlayRemove formItem icon"><span class="fas fa-times-circle" data-toggle="tooltip" title="Delete"></span></a>';
      overlayListHtml += '</td></tr>';
    }
    overlayListHtml += '</tbody>'

  } else {
    serviceArrTmp[activeItem].overlayList = [];
    overlayListHtml += '<p class="text-center">No overlay set.</p>';

  }

  $('#overlayList').html(overlayListHtml);
  $('.overlayMoveUp').click(moveOverlayClicked);
  $('.overlayMoveDown').click(moveOverlayClicked);
  $('.overlayEdit').click(editOverlayClicked);
  $('.overlayRemove').click(removeOverlayClicked);
  $('.radio_overlay_active').click(handleOverlayActiveClick);
  handleAutomationTypeLabel(serviceArrTmp[activeItem].automationType);

  showSaveWarning();
}

function handleAutomationTypeLabel(automation_type) {
  const tooltip_text = automation_type !== 'manual' ? 'To switch manually, change the triggering to Manual.' : 'For loop graphics playback, change the triggering to Automatic.';
  $('#automation_type_label').text(tooltip_text);
}

function handleOverlayActiveClick(event) {

  $('.radio_overlay_active').each((index, item) => {
    RadioInput(item).uncheck();
  });

  RadioInput(event.target).check();

  saveBtnCallback(true);
}

function RadioLabel(item) {

  let setActive = (is_active) => {
    is_active ? item.addClass('overlay_active') : item.removeClass('overlay_active');
  }

  let setText = (text) => {
    item.text(text);
  }

  return {
    setActive,
    setText
  }
}

function addOverlayClickedCallback() {
  activeOverlay = serviceArrTmp[activeItem].overlayList.length;
  added = true;
  const default_overlay = getDefaultOverlay();
  serviceArrTmp[activeItem].overlayList.push(default_overlay);
  setEditedOverlay();
  $('#editOverlayModal').modal('show');
}

function getDefaultOverlay() {
  return {
    "active": false,
    "coordSystem": "top_left",
    "pos_x": 10, "pos_y": 10,
    "imgPath": "",
    "imgName": "",
    "duration": 5,
    "scale": 1.0,
  };
}

async function setEditedOverlay() {
  createPositionList('#customPosition', serviceArrTmp[activeItem].overlayList[activeOverlay].coordSystem);
  $('#customPosX').val(serviceArrTmp[activeItem].overlayList[activeOverlay].pos_x);
  $('#customPosY').val(serviceArrTmp[activeItem].overlayList[activeOverlay].pos_y);
  let customScale = convertFloatToPercentage(serviceArrTmp[activeItem].overlayList[activeOverlay].scale);
  if (isNaN(customScale)) {
    customScale = 100;
  }
  $('#customScale').val(customScale);
  $('#customDuration').val(serviceArrTmp[activeItem].overlayList[activeOverlay].duration);

  // Create image list field
  await createImageList('#customImgPath', true, serviceArrTmp[activeItem].overlayList[activeOverlay].imgPath);
  customImagePathChanged();
}

function customImagePathChanged() {
  let options = $('#customImgPath option');
  let selectedIndex = $('#customImgPath')[0].selectedIndex;
  if (options.length === 0) {
    $('#customPreviewEditOverlay').html('');
    return;
  }
  let fileSrc = get_image_src(options[selectedIndex].getAttribute('value'));
  $('#imagePreviewEditOverlay').html('<img id="imageManagerPreview" src="' + fileSrc + '" />');
}

function get_image_src(fileSrc) {
  return '/local/camoverlay/api/image.cgi?path=' + encodeURIComponent(fileSrc) + '&_t=' + Date.now();
}

async function saveEditedOverlayCallback() {
  added = false;
  let imageList = await imageManager.getFileList();
  var image = findImageByPath(imageList, $('#customImgPath').val());
  serviceArrTmp[activeItem].overlayList[activeOverlay].coordSystem = $('#customPosition').val();
  serviceArrTmp[activeItem].overlayList[activeOverlay].pos_x = parseInt($('#customPosX').val());
  serviceArrTmp[activeItem].overlayList[activeOverlay].pos_y = parseInt($('#customPosY').val());
  serviceArrTmp[activeItem].overlayList[activeOverlay].imgPath = image.path;
  serviceArrTmp[activeItem].overlayList[activeOverlay].imgName = image.name;
  serviceArrTmp[activeItem].overlayList[activeOverlay].duration = parseInt($('#customDuration').val());
  serviceArrTmp[activeItem].overlayList[activeOverlay].scale = convertPercentageToFloat($('#customScale').val(), 100);

  showOverlayList();
}

function cancelEditedOverlayCallback() {
  if (added) {
    added = false;
    serviceArrTmp[activeItem].overlayList.pop();
  } else {
    serviceArrTmp[activeItem].overlayList[activeOverlay] = savedOverlay;
  }
  showOverlayList();
}

function moveOverlayClicked(event) {
  var targ = event.target || event.srcElement;
  var linkElement = $(targ).closest('a');
  var index = linkElement.attr('id').substring(11);
  var newIndex = parseInt(index) + 1;
  if (linkElement.hasClass('overlayMoveUp'))
    newIndex = parseInt(index) - 1;
  if (newIndex < 0 || newIndex >= serviceArrTmp[activeItem].overlayList.length)
    return false;
  var tmp = serviceArrTmp[activeItem].overlayList[index];
  serviceArrTmp[activeItem].overlayList[index] = serviceArrTmp[activeItem].overlayList[newIndex];
  serviceArrTmp[activeItem].overlayList[newIndex] = tmp;
  showOverlayList();
  return false;
}

function editOverlayClicked(event) {
  var targ = event.target || event.srcElement;
  var index = $(targ).closest('a').attr('id').substring(11);
  activeOverlay = index;
  savedOverlay = serviceArrTmp[activeItem].overlayList[activeOverlay];
  setEditedOverlay();
  $('#editOverlayModal').modal('show');
  return false;
}

function removeOverlayClicked(event) {
  var targ = event.target || event.srcElement;
  var index = $(targ).closest('a').attr('id').substring(13);
  serviceArrTmp[activeItem].overlayList.splice(index, 1);
  showOverlayList();
  return false;
}