function Capture() {
  let capture = null;

  let isWebcamSupported = () => {
    if (navigator.mediaDevices.getUserMedia) {
      return true;
    }
    return false;
  }

  let isScreenSharingSupported = () => {
    if (navigator.mediaDevices.getDisplayMedia) {
      return true;
    }
    return false;
  }

  let handleSharingErrors = (err_name, isWebcam) => {
    if (err_name === 'NotFunctionError') {
      return 'Not possible to share a mobile screen.';
    } else if (err_name === 'NotAllowedError') {
      if (isWebcam) {
        return 'Allow access to your computer camera.';
      }
      return 'Allow access to your computer screen.';
    } else if (err_name === 'NotFoundError') {
      return 'No source available.';
    } else if (err_name === 'NotReadableError') {
      return 'Not readable source.';
    } else if (err_name === 'SecurityError') {
      return 'Disabled in your browser.';
    }
    return 'Unable to start.';
  }

  let setScreen = async (fps) => {
    if (!isScreenSharingSupported()) {
      throw Error(handleSharingErrors('NotFunctionError', false));
    }

    try {
      capture = await navigator.mediaDevices.getDisplayMedia({ video: { frameRate: fps } });
    } catch (err) {
      console.error(err);
      throw Error(handleSharingErrors(err.name, false));
    }
    return capture;
  }

  let setWebcam = async (fps) => {
    if (!isWebcamSupported()) {
      throw Error(handleSharingErrors('NotFunctionError', true));
    }

    try {
      capture =  await navigator.mediaDevices.getUserMedia({ audio: false, video: { frameRate: fps, facingMode: "user" } });
    } catch (err) {
      console.error(err);
      throw Error(handleSharingErrors(err.name, true));
    }
    return capture;
  }

  let stop = () => {
    if (capture != undefined) {
      capture.getVideoTracks().forEach(item => {
        item.enabled && item.stop();
      });
    }
  }

  let onEnded = (callback) => $(capture.getVideoTracks()).off('ended').on('ended', () => callback());
  let set = (media_stream) => capture = media_stream;
  let get = () => capture;
  let getSettings = () => capture.getVideoTracks()[0].getSettings();
  let setFps = (fps) => {
    capture.getVideoTracks()[0].applyConstraints({ frameRate: fps });
  }

  return {
    set,
    setScreen,
    setWebcam,
    setFps,
    stop,
    onEnded,
    get,
    getSettings,
  }
}