var propertiesResponse = null;

$(document).ready(function() {

  initTooltips();
  initVideoChannel();
  initServiceId();

  // Fill form
  Switch().init('pipEnabled');

  $('#pipCustomName')
    .val(serviceArrTmp[activeItem].customName || '')
    .keyup(onCustomNameKeyUp)
    .attachValidation({
      pattern: '^[A-Za-z0-9 -]{0,}$',
      invalidMsg: 'Only chars A-Z, a-z, 0-9, white space and dash are allowed.',
      disableBtnId: 'saveBtn'
    });

  $('#pipPosX').val(serviceArrTmp[activeItem].pos_x);
  $('#pipPosY').val(serviceArrTmp[activeItem].pos_y);

  $('#pipFPS').val(serviceArrTmp[activeItem].fps || 1);
  $('.fpsMessage').toggle(!isArmv7hf());
  $('#pipCompression').val(serviceArrTmp[activeItem].compression || '50');
  $('#pipMaxSize').val(serviceArrTmp[activeItem].screenSize || 500);

  initPipSourceType();

  const resolutionId = '#pipResolution';
  const videoChannelId = '#pipVideoChannel';
  createResolutionList(resolutionId);
  onResolutionChange(resolutionId, videoChannelId);
  createPositionList('#pipPosition', serviceArrTmp[activeItem].coordSystem);
  createTriggeringElement('#pipAutomationType', serviceArrTmp[activeItem].automationType, serviceArrTmp[activeItem].schedule);
  onVideoChannelChange(resolutionId, videoChannelId, serviceArrTmp[activeItem].camera);
  createVideoChannelElement(videoChannelId, serviceArrTmp[activeItem].camera).then(async function(){

    let currentCameraResolutionObject = cameraResolutions[parseInt($('#pipResolution').val())];
    createCameraPreview($('#pipVideoChannel'), currentCameraResolutionObject, serviceArrTmp[activeItem].camera);

    // Callbacks for changes
    $('.form-control').change(formChangedCallback);

    $('#webcamSharingStart').click(sharing_start);
    $('#webcamSharingStop').click(sharing_stop);
    $('#pipMaxSize')
      .change(handle_change_max_size)
      .attachValidation({
        pattern: '^[1-9][0-9]{0,}$',
        invalidMsg: 'The value must not be empty or zero.',
        disableBtnId: 'webcamSharingStart'
      });
    $('#pipFPS')
      .change(handle_change_fps)
      .attachValidation({
        pattern: '^([1-9]|[12][0-9]|30)$',
        invalidMsg: 'The value must be 1 to 30.',
        disableBtnId: 'webcamSharingStart'
      });

    // Handle screen sharing state
    const running = sharingInterface.isRunning(serviceArrTmp[activeItem].id);
    if (running) {
      sharing_on();
    }

    $('#checkCameraConnection').on('click', function() {
      getProxyCameraData(100)
        .then(function () {

          showConnectionState(true, "Connection successful");
      }).catch(function(err) {

          showConnectionState(false, "Cannot connect to camera. Check the details of the remote camera.");
          console.log("Cannot connect to camera. Check the details of the remote camera.", err);
      });
    });

    if (serviceArrTmp[activeItem].source_type === "axis_camera") {
      if (serviceArrTmp[activeItem].camera_ip.length && serviceArrTmp[activeItem].camera_user.length && serviceArrTmp[activeItem].camera_pass.length) {
        getProxyCameraData(4000)
          .catch(function (err) {

            showConnectionState(false, "Cannot connect to camera. Check the details of the remote camera.");
            console.log("Cannot connect to camera. Check the details of the remote camera.", err);
          });
      } else {
        $("#cameraInfoForm").hide();
      }
    }

    formChangedCallback();
  });

  createScheduleBtn();
});

function initPipSourceType() {
  // Fill data
  $('#pipMjpegUrl').val(serviceArrTmp[activeItem].mjpeg_url);
  $('#pipCameraWidth').val(serviceArrTmp[activeItem].camera_width);
  $('#pipCameraHeight').val(serviceArrTmp[activeItem].camera_height);
  $('#pipCameraIP').val(serviceArrTmp[activeItem].camera_ip);
  $('#pipCameraPort').val(serviceArrTmp[activeItem].camera_port);
  $('#pipCameraUser').val(serviceArrTmp[activeItem].camera_user);
  $('#pipCameraPass').val(serviceArrTmp[activeItem].camera_pass);

  // Active tab
  $('#pipSourceType .change_tab#'+ serviceArrTmp[activeItem].source_type).tab('show');

  // Disable or enable FPS, Compression, Overlay size
  handleSourceTypeWithInputs(serviceArrTmp[activeItem].source_type);

  // Register event on tab click
  $('#pipSourceType .change_tab').off('click').on('click', function() {
    const tabId = $(this).attr('id');
    // Inputs display
    handleSourceTypeWithInputs(tabId);
    // Stop sharing if is not web camera (ws)
    if (tabId !== "ws") {
      sharing_stop();
    }

    showSaveWarning();
  });
}

function sharing_on() {
  $('#webcamSharingStart').hide();
  $('#webcamSharingStop').show();
  $('#stateCapture').show();
}

function sharing_off() {
  $('#webcamSharingStart').show();
  $('#webcamSharingStop').hide();
  $('#stateCapture').hide();
}

function handle_change_max_size() {
  const maxSize = Number($('#pipMaxSize').val());
  sharingInterface.setImageMaxSize(serviceArrTmp[activeItem].id, maxSize);
}

function handle_change_fps() {
  const fps = Number($('#pipFPS').val());
  sharingInterface.setFps(serviceArrTmp[activeItem].id, fps);
}

function handleSourceTypeWithInputs(sourceType) {
  const source_type_input_toggles = {
    "axis_camera": { fps: false, compression: false, max_size: true },
    "mjpeg_url": { fps: true, compression: true, max_size: true },
    "ws": { fps: false, compression: true, max_size: false },
  }
  serviceArrTmp[activeItem].source_type = sourceType;

  $('#pipFPS').attr('disabled', source_type_input_toggles[sourceType].fps);
  $('#pipCompression').attr('disabled', source_type_input_toggles[sourceType].compression);
  $('#pipMaxSize').attr('disabled', source_type_input_toggles[sourceType].max_size);
}


function handle_change_max_size() {
  const maxSize_element = $('#pipMaxSize');
  const min_value = maxSize_element.attr('min');
  const maxSize_value = Number(maxSize_element.val());
  if (maxSize_value >= min_value) {
    sharingInterface.setImageMaxSize(serviceArrTmp[activeItem].id, maxSize_value);
  }
}

function handle_change_fps() {
  const fps_element = $('#pipFPS');
  const min_val = fps_element.attr('min');
  const max_val = fps_element.attr('max');
  const fps_value = Number(fps_element.val());
  if (fps_value >= min_val && fps_value <= max_val) {
    sharingInterface.setFps(serviceArrTmp[activeItem].id, fps_value);
  }
}

async function sharing_start() {
  if (window.location.protocol != 'https:') {
    showMessage('warning', 'Screen sharing is available only on HTTPS');
    return;
  }
  if ($('#pipEnabled label[checked="true"]').length === 0) {
    showMessage('warning', 'Screen sharing service is currently disabled. Turn it on to start sharing.');
    return;
  }

  if (serviceArrTmp[activeItem].source_type !== serviceArr[activeItem].source_type) {
    showMessage('warning', 'Save your settings to start sharing your webcam.');
    return;
  }

  const url = new URL(window.location.origin);
  const ws_config = {
    url: 'wss://' + url.hostname + '/local/camoverlay/service/ws',
    name: 'pip'
  }
  const resolution = cameraResolutions[parseInt($('#pipResolution').val())];
  const imageMaxSize = Number($('#pipMaxSize').val());
  const fps = Number($('#pipFPS').val());
  const typeSharing = 'webcam';

  try {
    await sharingInterface.start(
      typeSharing,
      ws_config,
      serviceArrTmp[activeItem].id,
      resolution,
      imageMaxSize,
      false,
      fps,
      sharing_off,
    );
    sharing_on();

  } catch (err) {
    console.log(err);
    showMessage('danger', 'Web camera: ' + err.message);
  }
}

async function sharing_stop() {
  sharingInterface.stop(serviceArrTmp[activeItem].id);
}

function formChangedCallback() {
  serviceArrTmp[activeItem].customName = $('#pipCustomName').val();

  let currentCameraResolutionObject = cameraResolutions[parseInt($('#pipResolution').val())];
  serviceArrTmp[activeItem].width = currentCameraResolutionObject.width;
  serviceArrTmp[activeItem].height = currentCameraResolutionObject.height;
  serviceArrTmp[activeItem].coordSystem = $('#pipPosition').val();
  serviceArrTmp[activeItem].automationType = $('#pipAutomationType').val();
  serviceArrTmp[activeItem].pos_x = parseInt($('#pipPosX').val());
  serviceArrTmp[activeItem].pos_y = parseInt($('#pipPosY').val());
  serviceArrTmp[activeItem].fps = parseInt($('#pipFPS').val());
  if (serviceArrTmp[activeItem].fps < 1) {
    serviceArrTmp[activeItem].fps = 1;
    $('#pipFPS').val(serviceArrTmp[activeItem].fps);
  }

  serviceArrTmp[activeItem].compression = parseInt($('#pipCompression').val());
  serviceArrTmp[activeItem].screenSize = parseInt($('#pipMaxSize').val());

  serviceArrTmp[activeItem].mjpeg_url = $('#pipMjpegUrl').val().trim();

  serviceArrTmp[activeItem].camera_ip = $('#pipCameraIP').val();
  serviceArrTmp[activeItem].camera_port = parseInt($('#pipCameraPort').val());
  serviceArrTmp[activeItem].camera_user = $('#pipCameraUser').val();
  serviceArrTmp[activeItem].camera_pass = $('#pipCameraPass').val();

  if (propertiesResponse && $('#pipCameraViewArea').val() != null) {

    if (serviceArrTmp[activeItem].camera_view_area != $('#pipCameraViewArea').val()) {
      serviceArrTmp[activeItem].camera_view_area = $('#pipCameraViewArea').val();
      createCameraResolutionList(serviceArrTmp[activeItem].camera_view_area - 1, propertiesResponse);
    } else {
      serviceArrTmp[activeItem].camera_view_area = $('#pipCameraViewArea').val();
    }

    var cmResItem = $('#pipCameraResolution')[0];
    if(cmResItem.length > 0) {
      var res = cmResItem.options[cmResItem.selectedIndex].text;
      res = res.split('x');
      serviceArrTmp[activeItem].camera_width = parseInt(res[0].trim());
      serviceArrTmp[activeItem].camera_height = parseInt(res[1].trim());
    }
  }

  serviceArrTmp[activeItem].camera = saveVideoChannelValue(serviceArrTmp[activeItem].camera, $('#pipVideoChannel'));

  showSaveWarning();
  createServiceList(serviceArrTmp);
}

function waitForTimeout(timeout) {
  return new Promise(function (resolve) {
    setTimeout(resolve, timeout);
  });
}

function getProxyCameraData(time) {

  $('#cameraInfoForm').showSpinner('Loading data');

  return waitForTimeout(time)
    .then(function () {
      return getProxyCameraParam('Image.NbrOfConfigs');
    })
    .then(function(configResponse) {
      if (!configResponse.hasOwnProperty('image.nbrofconfigs')) {
        throw new Error('Connection error 1 - can\'t get params of proxy camera');
      }
      var configCount = parseInt(configResponse['image.nbrofconfigs']);
      var params = 'Properties.Image.Resolution,image.nbrofconfigs';
      for (var i = 0; i < configCount; i++) {
        params += ',Properties.Image.I' + i + '.Resolution,Image.I' + i + '.Name,Image.I' + i + '.Enabled';
      }
      return params;

    }).then(function(params) {
      return getProxyCameraParam(params);

    }).then(function(propsResponse) {
        if (!propsResponse.hasOwnProperty('properties.image.resolution')) {
          throw new Error('Connection error 2 - can\'t get params of proxy camera');
        }
        propertiesResponse = propsResponse;
        var configCount = parseInt(propsResponse['image.nbrofconfigs']);
        var selectedViewArea = createCameraViewAreaList(configCount, propertiesResponse); // Return selected view area number
        createCameraResolutionList(selectedViewArea, propertiesResponse);

    }).catch(function(err) {
      console.log(err);
      throw new Error("Connection error 4 - can't connect to proxy camera.");

    }).finally(function () {

      $('#cameraInfoForm').hideSpinner();
    });
}

function getProxyCameraParam(paramList) {
  return new Promise(function(resolve, reject) {
    let url = new URL('/local/camoverlay/api/proxy.cgi', window.location.origin);
    let urlParams = new URLSearchParams(url.search);
    urlParams.set('target_camera_ip', serviceArrTmp[activeItem].camera_ip);
    urlParams.set('target_camera_port', serviceArrTmp[activeItem].camera_port);
    urlParams.set('target_camera_path', '/axis-cgi/param.cgi');
    urlParams.set('target_camera_user', serviceArrTmp[activeItem].camera_user);
    urlParams.set('target_camera_pass', serviceArrTmp[activeItem].camera_pass);
    urlParams.set('action', 'list');
    urlParams.set('group', paramList);

    url.search = urlParams.toString();

    $.get(url, function(response) {
      resolve(parseParams(response));
    }).fail(function() {
      reject(new Error('Network error.'));
    });
  });
}

function parseResolutions(resString) {
  var resolutions = [];
  var resArr = resString.split(',');
  for (var i = 0; i < resArr.length; i++) {
    var resItems = resArr[i].split('x');
    if (resItems.length > 1) {
      resolutions.push({"width": + parseInt(resItems[0]), "height": + parseInt(resItems[1])});
    }
  }
  resolutions = resolutions.sort(function(a, b) {
    return b.width - a.width;
  });
  return resolutions;
}

function createCameraViewAreaList(viewAreaCount, cameraProperties) {
  var selected = -1;
  $('#pipCameraViewArea').empty();
  var resItem = $('#pipCameraViewArea')[0];
  for (var i = 0; i < viewAreaCount; i++) {
    if (cameraProperties['image.i' + i + '.enabled'] == 'yes') {
      if (i + 1 == serviceArrTmp[activeItem].camera_view_area) {
        selected = i;
      }
      resItem.options[resItem.options.length] = new Option(cameraProperties['image.i' + i + '.name'], i + 1);
    }
  }
  if (selected == -1) {
    selected = 0;
    if(resItem.options.length === 0){ // missing view areas in PTZ camera, add 'Default' value
      resItem.options[0] = new Option('Default', 1);
    }
    formChangedCallback();
  }
  resItem.selectedIndex = selected;
  return selected;
}

function createCameraResolutionList(viewArea, cameraProperties) {
  var resolutions = cameraProperties['properties.image.resolution'];
  if (cameraProperties.hasOwnProperty('properties.image.i' + viewArea + '.resolution')) {
    resolutions = cameraProperties['properties.image.i' + viewArea + '.resolution'];
  }
  resolutions = parseResolutions(resolutions);

  var selected = -1;
  $('#pipCameraResolution').empty();
  var resItem = $('#pipCameraResolution')[0];
  for (var i = 0; i < resolutions.length; i++) {
    if (resolutions[i].width == serviceArrTmp[activeItem].camera_width && resolutions[i].height == serviceArrTmp[activeItem].camera_height) {
      selected = i;
    }
    resItem.options[resItem.options.length] = new Option(resolutions[i].width + ' x ' + resolutions[i].height, i);
  }
  if (selected == -1) {
    selected = 0;
    formChangedCallback();
  }
  resItem.selectedIndex = selected;
  return selected;
}

function showConnectionState(state, text) {
  if (text !== undefined && text !== '') {
    if (state) {
      showMessage('success', text, false, 'pipConnectionStateSuccess');
      $("#cameraInfoForm").show();
    } else {
      showMessage('danger', text, false, 'pipConnectionStateError');
      $("#cameraInfoForm").hide();
    }
  }
}