$(document).ready(async function() {
  initTooltips();
  initVideoChannel();
  initServiceId();

  // Fill form
  Switch().init('screenSharingEnabled');

  $('#screenSharingCustomName')
    .val(serviceArrTmp[activeItem].customName || '')
    .keyup(onCustomNameKeyUp)
    .attachValidation({
      pattern: '^[A-Za-z0-9 -]{0,}$',
      invalidMsg: 'Only chars A-Z, a-z, 0-9, white space and dash are allowed.',
      disableBtnId: 'saveBtn'
    });

  $('#screenSharingPosX').val(serviceArrTmp[activeItem].pos_x);
  $('#screenSharingPosY').val(serviceArrTmp[activeItem].pos_y);

  $('#screenSharingFPS').val(serviceArrTmp[activeItem].fps || 1);
  $('.fpsMessage').toggle(!isArmv7hf());
  $('#screenSharingMaxSize').val(serviceArrTmp[activeItem].screenSize || 500);

  const resolutionId = '#screenSharingResolution';
  const videoChannelId = '#screenSharingVideoChannel';
  createResolutionList(resolutionId);
  onResolutionChange(resolutionId, videoChannelId);
  createPositionList('#screenSharingPosition', serviceArrTmp[activeItem].coordSystem);
  createTriggeringElement('#screenSharingAutomationType', serviceArrTmp[activeItem].automationType, serviceArrTmp[activeItem].schedule);
  onVideoChannelChange(resolutionId, videoChannelId, serviceArrTmp[activeItem].camera);
  await createVideoChannelElement(videoChannelId, serviceArrTmp[activeItem].camera);

  let currentCameraResolutionObject = cameraResolutions[parseInt($('#screenSharingResolution').val())];
  createCameraPreview($('#screenSharingVideoChannel'), currentCameraResolutionObject, serviceArrTmp[activeItem].camera);

  // Callbacks for changes
  $('.form-control').change(formChangedCallback);
  $('#screenSharingChange').click(sharing_change_screen);
  $('#screenSharingStart').click(sharing_start);
  $('#screenSharingStop').click(sharing_stop);
  $('#fullscreenOn').click(handle_fullscreen_on);
  $('#fullscreenOff').click(handle_fullscreen_off);
  $('#screenSharingMaxSize')
    .change(handle_change_max_size)
    .attachValidation({
      pattern: '^[1-9][0-9]{0,}$',
      invalidMsg: 'The value must not be empty or zero.',
      disableBtnId: 'screenSharingStart'
    });
  $('#screenSharingFPS')
    .change(handle_change_fps)
    .attachValidation({
      pattern: '^([1-9]|[12][0-9]|30)$',
      invalidMsg: 'The value must be 1 to 30.',
      disableBtnId: 'screenSharingStart'
    });

  // Handle screen sharing state
  const running = sharingInterface.isRunning(serviceArrTmp[activeItem].id);
  if (running) {
    handle_running_sharing();
  }

  formChangedCallback();
  createScheduleBtn();
});

function handle_running_sharing() {
  if (sharingInterface.getFullscreen(serviceArrTmp[activeItem].id)) {
    fullscreen_on();
  } else {
    fullscreen_off();
  }

  sharing_on();
}

function handle_change_max_size() {
  const maxSize_element = $('#screenSharingMaxSize');
  const min_value = maxSize_element.attr('min');
  const maxSize_value = Number(maxSize_element.val());
  if (maxSize_value >= min_value) {
    sharingInterface.setImageMaxSize(serviceArrTmp[activeItem].id, maxSize_value);
  }
}

function handle_change_fps() {
  const fps_element = $('#screenSharingFPS');
  const min_val = fps_element.attr('min');
  const max_val = fps_element.attr('max');
  const fps_value = Number(fps_element.val());
  if (fps_value >= min_val && fps_value <= max_val) {
    sharingInterface.setFps(serviceArrTmp[activeItem].id, fps_value);
  }
}

function fullscreen_on() {
  $('#fullscreen').prop('checked', true);
  $('#fullscreenOn').hide();
  $('#fullscreenOff').show();
}

function fullscreen_off() {
  $('#fullscreen').prop('checked', false);
  $('#fullscreenOff').hide();
  $('#fullscreenOn').show();
}

function handle_fullscreen_on() {
  sharingInterface.setFullscreen(serviceArrTmp[activeItem].id, true);
  fullscreen_on();
}

function handle_fullscreen_off() {
  sharingInterface.setFullscreen(serviceArrTmp[activeItem].id, false);
  fullscreen_off();
}

function sharing_on() {
  $('#screenSharingStart').hide();
  $('#screenSharingChange').show();
  $('#screenSharingStop').show();
  $('#fullscreenContainer').show();
  $('#stateCapture').show();
}

function sharing_off() {
  $('#screenSharingStart').show();
  $('#screenSharingChange').hide();
  $('#screenSharingStop').hide();
  $('#fullscreenContainer').hide();
  $('#stateCapture').hide();
}

async function sharing_start() {
  if (window.location.protocol != 'https:') {
    showMessage('warning', 'Screen sharing is available only on HTTPS');
    return;
  }
  if ($('#screenSharingEnabled label[checked="true"]').length === 0) {
    showMessage('warning', 'Screen sharing service is currently disabled. Turn it on to start sharing.');
    return;
  }

  const url = new URL(window.location.origin);
  const ws_config = {
    url: 'wss://' + url.hostname + '/local/camoverlay/service/ws',
    name: 'screen-sharing'
  }
  const resolution = cameraResolutions[parseInt($('#screenSharingResolution').val())];
  const imageMaxSize = Number($('#screenSharingMaxSize').val());
  const fullscreenOn = Boolean($('#fullscreen').is(':checked'));
  const fps = Number($('#screenSharingFPS').val());
  const typeSharing = 'screen';

  try {
    await sharingInterface.start(
      typeSharing,
      ws_config,
      serviceArrTmp[activeItem].id,
      resolution,
      imageMaxSize,
      fullscreenOn,
      fps,
      sharing_off,
    );
    sharing_on();

  } catch (err) {
    console.log(err);
    showMessage('danger', 'Screen Sharing: ' + err.message);
  }
}

async function sharing_change_screen() {
  sharing_stop();
  sharing_start();
}

async function sharing_stop() {
  sharingInterface.stop(serviceArrTmp[activeItem].id);
}

function formChangedCallback() {
  serviceArrTmp[activeItem].customName = $('#screenSharingCustomName').val();

  let currentCameraResolutionObject = cameraResolutions[parseInt($('#screenSharingResolution').val())];
  serviceArrTmp[activeItem].width = currentCameraResolutionObject.width;
  serviceArrTmp[activeItem].height = currentCameraResolutionObject.height;
  serviceArrTmp[activeItem].coordSystem = $('#screenSharingPosition').val();
  serviceArrTmp[activeItem].automationType = $('#screenSharingAutomationType').val();
  serviceArrTmp[activeItem].pos_x = parseInt($('#screenSharingPosX').val());
  serviceArrTmp[activeItem].pos_y = parseInt($('#screenSharingPosY').val());
  serviceArrTmp[activeItem].fps = parseInt($('#screenSharingFPS').val());
  serviceArrTmp[activeItem].screenSize = parseInt($('#screenSharingMaxSize').val());

  if (serviceArrTmp[activeItem].fps < 1) {
    serviceArrTmp[activeItem].fps = 1;
    $('#screenSharingFPS').val(serviceArrTmp[activeItem].fps);
  }

  serviceArrTmp[activeItem].camera = saveVideoChannelValue(serviceArrTmp[activeItem].camera, $('#screenSharingVideoChannel'));

  showSaveWarning();
  createServiceList(serviceArrTmp);
}