// shared js file for all acap aplications

// array of all acap for menu
var apps = ["CamStreamer", "CamSwitcher", "CamOverlay", "CamScripter"];

//if IE show message
var is_ie = /MSIE|Trident/.test(window.navigator.userAgent);
if (is_ie) {
  $('#test_js_working').text('Your browser doesn\'t support modern version of JavaScript. Please use current version of Chrome, Firefox, Safari or Edge.');
  $('#test_js_working').css('display','block');
  $('body > *:not(#test_js_working)').css('display', 'none');
}

// function - log error
function log_error(message,status){
  console.log('Error: ' + message + '(' + status + ')');
}

// function - get param from camera
function getParameter(paramList) {
  var promise = new Promise(function(resolve, reject) {
    $.get("/axis-cgi/param.cgi?action=list&group=" + paramList, function(data) {
      var params = {};
      var lines = data.split('\n');
      for (var i = 0; i < lines.length; i++) {
        var pos = lines[i].indexOf('=');
        if (pos != -1) {
          var name = lines[i].substring(0, pos).trim().toLowerCase();
          var value = lines[i].substring(pos + 1).trim();
          if (name.length && value.length) {
            params[name] = value;
          }
        }
      }
      resolve(params);
    });
  });
  return promise;
}

function setParameter(app_name, paramName, value) {
  var promise = new Promise(function(resolve, reject) {
    var dataJson = {};
    dataJson['action'] = 'update';
    dataJson[app_name + '.' + paramName] = value;

    $.ajax({
      method: "POST",
      cache: true,
      url: "/axis-cgi/param.cgi",
      data: dataJson
    }).done(function(msg) {
      try {
        if (msg.trim().toLowerCase() == 'ok')
          resolve();
        else
          reject();
      } catch(err) {
        reject();
      }

    });
  });
  return promise;
}

function getAcapPackage(name, serverUrl,installPage, defaultPage){

  getParameter("Brand.ProdNbr").then(function(dataModel) {
    getParameter("Properties.Firmware.Version").then(function(dataFirmware) {

      //get code, app_name, version, download_url
      $.get({
        url: serverUrl + '/index.php?p=ACAPPackage&a=getPackageInfo',
        dataType: "text",
        data: {
          camera_model: dataModel["brand.prodnbr"],
          camera_firmware: dataFirmware["properties.firmware.version"],
          app_name: name
        },
        success: function(response) {
          var data = JSON.parse(response);
          if(data.code === 200){
            $('#install_button').on('click', function(){
              var name = data.download_url.split("/");
              downloadApp(serverUrl, data.app_name, name[name["length"]-1], data.download_url, 0, installPage, defaultPage);
            });
          }else{
            renderNotSupportedCamera(name, serverUrl);
          }
        },
        error: function(jqXHR, textStatus, errorThrown){
          renderErrorPrepare(name, serverUrl);
          log_error('Something went wrong with getting information about compatible application for your camera, please try again.');
        }
      });
    });
  });
}

function prepareUpdate(serverUrl, latestACAPVersion, currentVersion){
  //prepare changelog data and insert to modal, then open modal
  var modal = $('#releaseNotesInfo');
  var modal_body = modal.find('.modal-body');
  var modal_title = modal.find('.modal-title');
  modal_title.text('Release notes for ' + app_name + ' App version ' + latestACAPVersion);
  modal.modal('show');
  modal_body.text('');

  //get release notes for latest acap version
  $.get({
    url: serverUrl + '/index.php?p=ACAPPackage&a=getReleaseNotes',
    data: {
      app_name: app_name,
      target_version: latestACAPVersion,
      current_version: currentVersion
    },
    success: function(response) {

      if(response.code !== 'OK'){
        modal_body.html('<div id="getReleaseNotesInfoError" class="alert alert-danger">' + 'Failed to get release notes.' + '</div>');
        log_error(response.message, response.code);
      } else {

        if(response.data.length === 0){
          modal_body.html('<div id="getReleaseNotesInfoError" class="alert alert-info">' + 'No release notes for this update.' + '</div>');
        }

        var note_row_template = '<div class="row"><div class="col-3">' + app_name + ' App %VERSION%<br><span class="text-nowrap">%DATE%</span></div><div class="col">%CONTENT%</div></div>';
        response.data.forEach(function (note) {
          modal_body.append(note_row_template.replace('%VERSION%', note.version).replace('%DATE%', note.release_date).replace('%CONTENT%', note.content));
        });

      }
    },
    error: function(response){
      modal_body.html('<div id="getReleaseNotesInfoError" class="alert alert-danger">' + 'Failed to get release notes.' + '</div>');
      log_error(response.statusText, response.status);
    }
  });

}

function getUpdate(name, serverUrl, installPage, defaultPage){

  var update = 1;

  getParameter("Brand.ProdNbr").then(function(dataModel) {
    getParameter("Properties.Firmware.Version").then(function(dataFirmware) {

      //get code, app_name, version, download_url
      $.get({
        url: serverUrl + '/index.php?p=ACAPPackage&a=getPackageInfo',
        dataType: "text",
        data: {
          camera_model: dataModel["brand.prodnbr"],
          camera_firmware: dataFirmware["properties.firmware.version"],
          app_name: name
        },
        success: function(response) {
          var data = JSON.parse(response);
          if(data.code === 200) {
              var name = data.download_url.split("/");
              var package_name = name[name["length"]-1];
              var app_id = name[name["length"]-1].split("_");
              downloadApp(serverUrl, data.app_name, name[name["length"]-1], data.download_url, update, installPage, defaultPage);
          } else {
            log_error('Something went wrong with getting compatible package for your camera, please try again.')
          }
        },
        error: function(jqXHR, textStatus, errorThrown){
          log_error('Something went wrong with getting information about compatible application for your camera, please try again.');
        }
      });
    });
  });

}

function downloadFile(serverUrl, url, app_name, update){
  if(update) {
    renderUpdateProcess(app_name, serverUrl);
  } else {
    renderInstallAndDownloadProcess(app_name, serverUrl);
  }

  var promise = new Promise(function(resolve, reject) {
    var oReq = new XMLHttpRequest();
    oReq.open("GET", url, true);
    oReq.responseType = "blob";
    oReq.onload = function (oEvent) {
      var blob = oReq.response;
      if(oReq.status == 200){
        resolve(blob);
      }else{
        reject();
        log_error("Error - downloading file.", oReq.status);
      }
    };
    oReq.addEventListener("error", function(){
      reject();
      log_error("Error - downloading file.", oReq.status);
    });

    oReq.send();
  });
  return promise;
}

function downloadApp(serverUrl, app_name, file_name, url, update, installPage, defaultPage){
  downloadFile(serverUrl, url, app_name, update).then(function(data){
    installApp(serverUrl, app_name, file_name, data, update, installPage, defaultPage);
  }, function(error){
    renderErrorDownload(app_name, serverUrl);
  });
}

function installApp(serverUrl, app_name, file_name, blob, update, installPage, defaultPage){
  var form = $('#installForm')[0];
  var formData = new FormData(form);
  formData.append("packfil", blob, file_name);

  $.post({
    url: '/axis-cgi/applications/upload.cgi',
    data: formData,
    contentType: false,
    processData: false
  }).done(function(){
    if(update){
      renderSuccessfullUpdate(app_name, serverUrl, defaultPage);
    }else {
      renderSuccessfullInstallation(app_name, serverUrl, defaultPage);
    }
  }).fail(function(response, status, error){
    renderErrorInstallation(app_name, serverUrl);
    log_error(error, status);
  });
}

function renderErrorPrepare(app_name, serverUrl){
  var content = '<div class="card card-empty text-center" id="download_error_' + app_name + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-exclamation-triangle text-danger align-middle"></span></h1>' +
    '<p class="align-middle">Something went wrong with preparing <a class="text" target="_blank" href="' + serverUrl + '/' + app_name.toLowerCase() + '-about">' + app_name + ' App</a></p>' +
    '</blockquote>' +
    '<p>Please contact our support team on <a href="mailto:support@camstreamer.com">support@camstreamer.com</a>.</p>' +
    '<a href="mailto:support@camstreamer.com" class="btn btn-sm btn-primary pt-2 pb-2 pl-4 pr-4"><span class="fas fa-life-ring mr-2"></span>Contact support</a>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);
}

function renderNotSupportedCamera(app_name, serverUrl){
  var content = '<div class="card card-empty text-center" id="download_error_' + app_name + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-exclamation-triangle text-danger align-middle"></span></h1>' +
    '<p class="align-middle">Your camera does not support <a class="text" target="_blank" href="' + serverUrl + '/' + app_name.toLowerCase() + '-about">' + app_name + ' App</a></p>' +
    '</blockquote>' +
    '<br><p>Check compatible cameras <a target="_blank" href="' + serverUrl + '/download-app-all-supported-cameras#' + app_name.toLowerCase() + '-download">here</a>.</p>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);
}

function renderErrorDownload(app_name, serverUrl){
  var content = '<div class="card card-empty text-center" id="download_error_' + app_name + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-exclamation-triangle text-danger align-middle"></span></h1>' +
    '<p class="align-middle">Error in downloading of <a class="text" target="_blank" href="' + serverUrl + '/' + app_name.toLowerCase() + '-about">' + app_name + ' App</a></p>' +
    '</blockquote>' +
    '<p>Please contact our support team on <a href="mailto:support@camstreamer.com">support@camstreamer.com</a>.</p>' +
    '<a href="mailto:support@camstreamer.com" class="btn btn-sm btn-primary pt-2 pb-2 pl-4 pr-4"><span class="fas fa-life-ring mr-2"></span>Contact support</a>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);
}

function renderErrorInstallation(app_name, serverUrl){
  var content = '<div class="card card-empty text-center" id="installation_error_' + app_name + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-exclamation-triangle text-danger align-middle"></span></h1>' +
    '<p class="align-middle">Error in installation of <a class="text" target="_blank" href="' + serverUrl + '/' + app_name.toLowerCase() + '-about">' + app_name + ' App</a></p>' +
    '</blockquote>' +
    '<p>Please contact our support team on <a href="mailto:support@camstreamer.com">support@camstreamer.com</a>.</p>' +
    '<a href="mailto:support@camstreamer.com" class="btn btn-sm btn-primary pt-2 pb-2 pl-4 pr-4"><span class="fas fa-life-ring mr-2"></span>Contact support</a>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);
}

function renderUpdateProcess(app_name, serverUrl){
  $("#" + app_name + " .badge").text('updating').removeClass('badge-dark').addClass('badge-info'); //change badge in menu
  $('div.container[role="main"]').html(''); //empty main container
  $('div.container[role="main"]').append('<div id="content_installation" class="mt-4"></div>'); //fill it with element
  var content_progress = '<div class="card card-empty text-center" id="installation_progress_' + app_name + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-cog fa-spin text-primary align-middle"></span></h1>' +
    '<p class="align-middle">Updating <a class="text" target="_blank" href="' + serverUrl + '/' + app_name.toLowerCase() + '-about">' + app_name + ' App</a></p>' +
    '</blockquote>' +
    '<p>Please wait until the end of this update.</p>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content_progress);
}

function renderInstallAndDownloadProcess(app_name, serverUrl){
  var content = '<div class="card card-empty text-center" id="installation_success_' + app_name + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-cog fa-spin text-primary align-middle"></span></h1>' +
    '<p class="align-middle">Downloading & installing <a class="text" target="_blank" href="' + serverUrl + '/' + app_name.toLowerCase() + '-about">' + app_name + ' App</a></p>' +
    '</blockquote>' +
    '<p>Please wait till end of this installation.</p>' +
    '</div>' +
    '</div>' +
    '<form id="installForm" style="display:none;" action="" method="POST" enctype="multipart/form-data">' +
    '</form>';

  $('#content_installation').html(content);
}

function renderSuccessfullUpdate(app_name, serverUrl, defaultPage){
  $("#" + app_name + " .badge").text('updated').removeClass('badge-info').addClass('badge-success'); //change badge status in menu

  var link = '/local/' + app_name.toLowerCase() + '/' + defaultPage;
  var content = '<div class="card card-empty text-center" id="installation_success_' + app_name + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-check-circle text-success align-middle"></span></h1>' +
    '<p class="align-middle">Update of <a class="text" target="_blank" href="' + serverUrl + '/' + app_name.toLowerCase() + '-about">' + app_name + ' App</a> was successfull!</p>' +
    '</blockquote>' +
    '<p>Feel free to contact us via help menu if you have any questions.  Enjoy our awesome app!</p>' +
    '<a href="' + link + '" class="btn btn-sm btn-primary pt-2 pb-2 pl-4 pr-4"><span class="fas fa-archive mr-2"></span>Open app</a>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);
}

function renderSuccessfullInstallation(app_name, serverUrl, defaultPage){
  var link = '/local/' + app_name.toLowerCase() + '/' + defaultPage;
  var content = '<div class="card card-empty text-center" id="installation_success_' + app_name + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-check-circle text-success align-middle"></span></h1>' +
    '<p class="align-middle">Installation of <a class="text" target="_blank" href="' + serverUrl + '/' + app_name.toLowerCase() + '-about">' + app_name + ' App</a> was successfull!</p>' +
    '</blockquote>' +
    '<p>Feel free to contact us via help menu if you have any questions.  Enjoy our awesome app!</p>' +
    '<a href="' + link + '" class="btn btn-sm btn-primary pt-2 pb-2 pl-4 pr-4"><span class="fas fa-archive mr-2"></span>Open app</a>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);
}

function getAppList() {
  var promise = new Promise(function(resolve, reject) {
    $.get('/axis-cgi/applications/list.cgi').then(function(xml) {
      var result = xmlToJSON.parseXML(xml);
      var apps = [];
      for (var i = 0; i < result.reply[0]["application"].length; i++) {
        apps.push(result.reply[0]["application"][i]["_attr"]);
      }
      resolve(apps);
    }, reject);
  }.bind(this));
  return promise;
}

function renderMenu(app, serverUrl) {
  if(app === '' || app === undefined){
    app = app_name;
  }

  getAppList().then(function(data) {
    var menu = [];
    if (data != null) {
      for (var i = 0; i < data.length; i++) {
        var menuItem = "";
        var ConfigurationPage = "";
        if (data[i]["ConfigurationPage"] === undefined) {
          ConfigurationPage = "";
        } else {
          ConfigurationPage = data[i]["ConfigurationPage"]["_value"];
        }
        var version = String(data[i]["Version"]["_value"]).replace("-", ".");
        var niceNameApp = data[i]["NiceName"]["_value"];
        var niceName = niceNameApp.replace(' App','');

        menuItem += '<a id="' + niceName + '" class="';
        if (niceName === app) {
          menuItem += 'active ';
        }
        menuItem += 'mr-2 nav-link btn-menu" href="/' + ConfigurationPage;
        menuItem += '">' + niceNameApp + '<span class="ml-2 badge badge-dark" style="font-size: 0.6rem;">';
        menuItem += version + '</span></a>';

        menu[niceName] = menuItem;
      }
    }

    var insertHTML = "";
    for (var o = 0; o < apps.length; o++) {
      // Fill not install apps
      if (menu[apps[o]] === undefined) {
        insertHTML += '<a id="' + apps[o] + '" server-url="' + serverUrl + '" class="';
        if (apps[o] === app) {
          insertHTML += 'active ';
        }
        insertHTML += 'mr-2 nav-link btn-menu install_package" href="javascript:void(0);">' + apps[o] + " App" + '<span class="ml-2 badge badge-dark" style="font-size: 0.6rem;">start trial</span></a>';
      } else {
        insertHTML += menu[apps[o]];
      }
    }

    $('#main_menu').html(insertHTML);

    $('.install_package').on('click', function() {
      notInstalledApp($(this).attr('id'), $(this).attr('server-url'));
    });
  });
}

function notInstalledApp(name, serverUrl) {
  var oldUrl = location.pathname;
  oldUrl = oldUrl.split('/');
  var newUrl = '/local/' + encodeURIComponent(oldUrl[2].toLowerCase()) + '/shared/installApp.html?app='+ encodeURIComponent(name) + '&serverUrl=' + encodeURIComponent(serverUrl);
  location.href = newUrl;
}

function findGetParameter(parameterName) {
  var result = null,
    tmp = [];
  location.search
    .substr(1)
    .split("&")
    .forEach(function (item) {
      tmp = item.split("=");
      if (tmp[0] === parameterName) result = decodeURIComponent(tmp[1]);
    });
  return result;
}

function renderInstallApp(app_name, serverUrl, installPage, defaultPage) {
  const nice_name = app_name + " App";
  document.title = nice_name;
  renderStyles(app_name);

  var content = '<div class="card card-empty text-center" id="install_div_' + app_name + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><img src="logo_CamStreamer.png" width="30" style="filter:invert(100%);" /></h1>' +
    '<p>Install trial version of <a class="text" target="_blank" href="' + serverUrl + '/' + app_name.toLowerCase() + '-about">' + nice_name + '</a></p>' +
    '</blockquote>' +
    '<p>Install trial version of our awesome ' + nice_name + ' in one click to your camera.</p>' +
    '<button type="button" id="install_button" class="btn btn-sm btn-primary pt-2 pb-2 pl-4 pr-4"><span class="fas fa-download mr-2"></span>Install</button>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);

  getAcapPackage(app_name, serverUrl, installPage, defaultPage);
}

function renderStyles(app_name){
  app = app_name.toLowerCase();
  $('head').append('<link rel="stylesheet" href="bt_' + app + '.css">');
}

// Replace server url in links
function replaceServerUrl(serverUrl) {
  $('.serverURL').each(function() {
    var replaced = false;
    if (this.getAttribute('href')) {
      this.setAttribute('href', serverUrl + '/' + this.getAttribute('href').split('/').slice(3).join('/'));
      replaced = true;
    }
    if (this.getAttribute('src')) {
      this.setAttribute('src', serverUrl + '/' + this.getAttribute('src').split('/').slice(3).join('/'));
      replaced = true;
    }
    if (this.getAttribute('original')) {
      this.setAttribute('original', serverUrl + '/' + this.getAttribute('original').split('/').slice(3).join('/'));
      replaced = true;
    }

    if (!replaced)
    {
      console.error('replaceServerUrl: undefined item type: ' + this.tagName + ', content: ' + this.textContent);
    }
  });
}